Attribute VB_Name = "SmartEdge"
' ----------------------------------------------------
' Module SmartEdge
'
' Purpose: Set/Unset different border styles
'          for any window.
'
' Version: 1.2
' Date: 22 April 2001
' (Update: 22, Feb, 2005)
'
' Author: Dipak Auddy
' E-mail: dauddy@gmail.com
' ----------------------------------------------------
Option Explicit
'Different border styles we can apply to any window:
Public Enum sedBorderStyle
    sedNone
    sedSunken
    sedSunkenOuter
    sedRaised
    sedRaisedInner
    sedBump
    sedEtched
End Enum

Private Enum sedBorderWidth
    sbwNone
    sbwSingle
    sbwDouble
End Enum

Private Const SED_OLDPROC          As String = "SED_OLDPROC"
Private Const SED_OLDGWLSTYLE      As String = "SED_OLDGWLSTYLE"
Private Const SED_OLDGWLEXSTYLE    As String = "SED_OLDGWLEXSTYLE"
Private Const SED_BORDERS          As String = "SED_BORDERS"

'API declarations:
Private Const WM_NCPAINT           As Long = &H85

Private Const SWP_FRAMECHANGED     As Long = &H20
Private Const SWP_NOMOVE           As Long = &H2
Private Const SWP_NOOWNERZORDER    As Long = &H200
Private Const SWP_NOSIZE           As Long = &H1
Private Const SWP_NOZORDER         As Long = &H4

Private Const BDR_RAISED           As Long = &H5
Private Const BDR_RAISEDINNER      As Long = &H4
Private Const BDR_RAISEDOUTER      As Long = &H1
Private Const BDR_SUNKEN           As Long = &HA
Private Const BDR_SUNKENINNER      As Long = &H8
Private Const BDR_SUNKENOUTER      As Long = &H2

Private Const BF_LEFT              As Long = &H1
Private Const BF_RIGHT             As Long = &H4
Private Const BF_TOP               As Long = &H2
Private Const BF_BOTTOM            As Long = &H8
Private Const BF_RECT              As Long = (BF_LEFT Or BF_TOP Or BF_RIGHT Or BF_BOTTOM)

Private Const GWL_WNDPROC          As Long = (-4)
Private Const GWL_STYLE            As Long = (-16)
Private Const GWL_EXSTYLE          As Long = (-20)

Private Const WS_THICKFRAME        As Long = &H40000
Private Const WS_BORDER            As Long = &H800000
Private Const WS_EX_WINDOWEDGE     As Long = &H100
Private Const WS_EX_CLIENTEDGE     As Long = &H200
Private Const WS_EX_STATICEDGE     As Long = &H20000

Private Type RECT
    Left                               As Long
    Top                                As Long
    Right                              As Long
    Bottom                             As Long
End Type
Private Declare Function GetWindowDC Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function ReleaseDC Lib "user32" (ByVal hWnd As Long, _
                                                 ByVal hDC As Long) As Long
Private Declare Function GetWindowRect Lib "user32" (ByVal hWnd As Long, _
                                                     lpRect As RECT) As Long
Private Declare Function DrawEdge Lib "user32" (ByVal hDC As Long, _
                                                qrc As RECT, _
                                                ByVal edge As Long, _
                                                ByVal grfFlags As Long) As Long
Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, _
                                                                            ByVal nIndex As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, _
                                                                            ByVal nIndex As Long, _
                                                                            ByVal dwNewLong As Long) As Long
Private Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, _
                                                    ByVal hWndInsertAfter As Long, _
                                                    ByVal X As Long, _
                                                    ByVal Y As Long, _
                                                    ByVal cx As Long, _
                                                    ByVal cy As Long, _
                                                    ByVal wFlags As Long) As Long
Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, _
                                                                              ByVal hWnd As Long, _
                                                                              ByVal Msg As Long, _
                                                                              ByVal wParam As Long, _
                                                                              ByVal lParam As Long) As Long
Private Declare Function GetProp Lib "user32" Alias "GetPropA" (ByVal hWnd As Long, _
                                                                ByVal lpString As String) As Long
Private Declare Function SetProp Lib "user32" Alias "SetPropA" (ByVal hWnd As Long, _
                                                                ByVal lpString As String, _
                                                                ByVal hData As Long) As Long
Private Declare Function RemoveProp Lib "user32" Alias "RemovePropA" (ByVal hWnd As Long, _
                                                                      ByVal lpString As String) As Long

' ------------------------------------------------------------------------
' Sub ApplyBorderStyle()
'
'      This function allows you to modify the border style of any window
'      Example: ApplyBorderStyle(Picture1.Hwnd,sedRaisedInner)
'
Public Sub ApplyBorderStyle(ByVal lngHWnd As Long, _
                            ByVal eBorderStyle As sedBorderStyle)

Dim lRet As Long

    'Check whether the window was already subclassed
    'and get the original windowproc...
    lRet = GetProp(lngHWnd, SED_OLDPROC)
    If lRet <> 0 Then
        'Unsubclass the window...
        SetWindowLong lngHWnd, GWL_WNDPROC, lRet
    Else 'NOT LRET...
        'Store the window style (only the first time we subclass the window)...
        SetProp lngHWnd, SED_OLDGWLSTYLE, GetWindowLong(lngHWnd, GWL_STYLE)
        SetProp lngHWnd, SED_OLDGWLEXSTYLE, GetWindowLong(lngHWnd, GWL_EXSTYLE)
    End If
    'Change to the window border that best suits our drwaing requirements...
    pSetBorder lngHWnd, eBorderStyle
    'Subclass the window...
    lRet = SetWindowLong(lngHWnd, GWL_WNDPROC, AddressOf pWindowProc)
    'Store the original windowproc and the new border style...
    SetProp lngHWnd, SED_OLDPROC, lRet
    SetProp lngHWnd, SED_BORDERS, CLng(eBorderStyle)
    'Refresh the window (this forces Windows to send a WM_NCPAINT message)...
    SetWindowPos lngHWnd, 0, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE Or SWP_NOOWNERZORDER Or SWP_NOZORDER Or SWP_FRAMECHANGED

End Sub

' ------------------------------------------------------------------------
' Sub pDrawBorder()
'
'      The purpose of this procedure is to draw the window edge.
'      Called by pWindowProc when receives the message WM_NCPAINT.
'
Private Sub pDrawBorder(ByVal lngHWnd As Long, _
                        ByVal wParam As Long, _
                        ByVal lBorderType As sedBorderStyle)

Dim lMode As Long
Dim hDC   As Long
Dim Rec   As RECT

    'There's no drawing needed when there's no border assigned...
    If lBorderType = sedNone Then
        Exit Sub
    End If '<:-) Structure Expanded.
    'Get a device context for this window handle...
    hDC = GetWindowDC(lngHWnd)
    'Get the RECT that contains the window...
    Call GetWindowRect(lngHWnd, Rec)
    
    With Rec
        .Right = .Right - .Left
        .Bottom = .Bottom - .Top
        .Left = 0
        .Top = 0
        'Choose the drawing flags based on the selected border style...
    End With 'Rec
    lMode = 0
    Select Case lBorderType
    Case sedRaised
        lMode = BDR_RAISED
    Case sedRaisedInner
        lMode = BDR_RAISEDINNER
    Case sedSunken
        lMode = BDR_SUNKEN
    Case sedSunkenOuter
        lMode = BDR_SUNKENOUTER
    Case sedEtched
        lMode = BDR_SUNKENOUTER Or BDR_RAISEDINNER
    Case sedBump
        lMode = BDR_SUNKENINNER Or BDR_RAISEDOUTER
    End Select
    'Draw the window border by using the API DrawEdge...
    Call DrawEdge(hDC, Rec, lMode, BF_RECT)
    'Release the device context...
    Call ReleaseDC(lngHWnd, hDC)

End Sub

' ------------------------------------------------------------------------
' Sub pSetBorder()
'
'      Different border styles have different widths.
'      To draw the border edge we first need to make sure that
'      the window border width will be correct.
'
Private Sub pSetBorder(ByVal lngHWnd As Long, _
                       ByVal eBorderStyle As sedBorderStyle)

Dim pWidth As sedBorderWidth

    'Depending on the border style we want to draw,
    'we need a width of 0, 1 or 2 pixels...
    Select Case eBorderStyle
    Case sedNone
        pWidth = sbwNone
    Case sedRaised
        pWidth = sbwDouble
    Case sedRaisedInner
        pWidth = sbwSingle
    Case sedSunken
        pWidth = sbwDouble
    Case sedSunkenOuter
        pWidth = sbwSingle
    Case sedEtched
        pWidth = sbwDouble
    Case sedBump
        pWidth = sbwDouble
    End Select
    'Change the border style depending on the width...
    Select Case pWidth
    Case sbwNone
        pWinStyleNeg lngHWnd, GWL_STYLE, WS_BORDER Or WS_THICKFRAME
        pWinStyleNeg lngHWnd, GWL_EXSTYLE, WS_EX_STATICEDGE Or WS_EX_CLIENTEDGE Or WS_EX_WINDOWEDGE
    Case sbwSingle
        pWinStyleNeg lngHWnd, GWL_STYLE, WS_BORDER Or WS_THICKFRAME
        pWinStyleNeg lngHWnd, GWL_EXSTYLE, WS_EX_CLIENTEDGE Or WS_EX_WINDOWEDGE
        pWinStyleAdd lngHWnd, GWL_EXSTYLE, WS_EX_STATICEDGE
    Case sbwDouble
        pWinStyleNeg lngHWnd, GWL_STYLE, WS_BORDER Or WS_THICKFRAME
        pWinStyleNeg lngHWnd, GWL_EXSTYLE, WS_EX_STATICEDGE Or WS_EX_WINDOWEDGE
        pWinStyleAdd lngHWnd, GWL_EXSTYLE, WS_EX_CLIENTEDGE
    End Select
    'Refresh the window (sends message WM_NCPAINT)...
    SetWindowPos lngHWnd, 0, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE Or SWP_NOOWNERZORDER Or SWP_NOZORDER Or SWP_FRAMECHANGED

End Sub

' ------------------------------------------------------------------------
' Sub pWindowProc()
'
'      Once a window has been subclassed, this procedure will
'      receive all messages instead of its original windowproc.
'      We can now filter the WM_NCPAINT message and use our own
'      functions to draw the window edge.
'
Private Sub pWindowProc(ByVal lngHWnd As Long, _
                        ByVal uMsg As Long, _
                        ByVal wParam As Long, _
                        ByVal lParam As Long)

    Select Case uMsg 'Select the message uMsg and only modify WM_NCPAINT...
    Case WM_NCPAINT 'Message sent to a window when its non-client area needs to be re-drawn ...
        'Call our own drawing function...
        pDrawBorder lngHWnd, wParam, GetProp(lngHWnd, SED_BORDERS)
    Case Else
        'All other messages should be sent to the original windowproc...
        CallWindowProc GetProp(lngHWnd, SED_OLDPROC), lngHWnd, uMsg, wParam, lParam
    End Select

End Sub

' Helper func.
Private Sub pWinStyleAdd(ByVal lngHWnd As Long, _
                         ByVal lStyle As Long, _
                         ByVal lFlags As Long)

'Add flags to the window style

    SetWindowLong lngHWnd, lStyle, GetWindowLong(lngHWnd, lStyle) Or lFlags

End Sub

' Helper funct.
Private Sub pWinStyleNeg(ByVal lngHWnd As Long, _
                         ByVal lStyle As Long, _
                         ByVal lFlags As Long)

'Remove flags from the window style

    SetWindowLong lngHWnd, lStyle, GetWindowLong(lngHWnd, lStyle) And Not lFlags

End Sub

' ------------------------------------------------------------------------
' Sub RestoreBorderStyle()
'
'      This function restores a subclassed window to its original status
'      Example: RestoreBorderStyle(Picture1.Hwnd)
'
Public Sub RestoreBorderStyle(ByVal lngHWnd As Long)

Dim lRet As Long

    'Get the original windowproc for this window...
    lRet = GetProp(lngHWnd, SED_OLDPROC)
    If lRet <> 0 Then
        'Unsubclass the window by assigning the original windowproc...
        lRet = SetWindowLong(lngHWnd, GWL_WNDPROC, lRet)
        'Restore the original window styles...
        SetWindowLong lngHWnd, GWL_STYLE, GetProp(lngHWnd, SED_OLDGWLSTYLE)
        SetWindowLong lngHWnd, GWL_EXSTYLE, GetProp(lngHWnd, SED_OLDGWLEXSTYLE)
        'Refresh the window (sends message WM_NCPAINT)...
        SetWindowPos lngHWnd, 0, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE Or SWP_NOOWNERZORDER Or SWP_NOZORDER Or SWP_FRAMECHANGED
        'Remove all stored information for this window...
        RemoveProp lngHWnd, SED_OLDPROC
        RemoveProp lngHWnd, SED_OLDGWLSTYLE
        RemoveProp lngHWnd, SED_OLDGWLEXSTYLE
        RemoveProp lngHWnd, SED_BORDERS
    End If

End Sub
